@@ -18,12 +18,15 @@
 #include "mozilla/Attributes.h"
 #include "mozilla/FontPropertyTypes.h"
 #include "mozilla/MemoryReporting.h"
+#include "mozilla/Mutex.h"
 #include "mozilla/RefPtr.h"
-#include "mozilla/ServoStyleConsts.h"
+#include "mozilla/RWLock.h"
 #include "mozilla/TypedEnumBits.h"
 #include "mozilla/UniquePtr.h"
 #include "mozilla/gfx/MatrixFwd.h"
 #include "mozilla/gfx/Point.h"
+#include "mozilla/gfx/2D.h"
+#include "mozilla/intl/UnicodeScriptCodes.h"
 #include "nsCOMPtr.h"
 #include "nsColor.h"
 #include "nsTHashMap.h"
@@ -37,26 +40,16 @@
 #include "nsString.h"
 #include "nsTArray.h"
 #include "nsTHashtable.h"
-#include "nsUnicodeScriptCodes.h"
 #include "nscore.h"
+#include "DrawMode.h"
 
-// Only required for function bodys
-#include <stdlib.h>
-#include <string.h>
-#include <algorithm>
-#include "mozilla/Assertions.h"
-#include "mozilla/HashFunctions.h"
-#include "mozilla/ServoUtils.h"
-#include "mozilla/gfx/2D.h"
+// Only required for function bodies
 #include "gfxFontEntry.h"
 #include "gfxFontFeatures.h"
-#include "gfxFontUtils.h"
-#include "gfxPlatform.h"
-#include "nsAtom.h"
-#include "nsDebug.h"
-#include "nsMathUtils.h"
 
 class gfxContext;
+class gfxGraphiteShaper;
+class gfxHarfBuzzShaper;
 class gfxGlyphExtents;
 class gfxMathTable;
 class gfxPattern;
@@ -67,7 +60,6 @@ class gfxTextRun;
 class nsIEventTarget;
 class nsITimer;
 struct gfxTextRunDrawCallbacks;
-enum class DrawMode : int;
 
 namespace mozilla {
 class SVGContextPaint;
@@ -103,7 +95,8 @@ struct gfxFontStyle {
   gfxFontStyle(FontSlantStyle aStyle, FontWeight aWeight, FontStretch aStretch,
                gfxFloat aSize, const FontSizeAdjust& aSizeAdjust,
                bool aSystemFont, bool aPrinterFont, bool aWeightSynthesis,
-               bool aStyleSynthesis, uint32_t aLanguageOverride);
+               bool aStyleSynthesis, bool aSmallCapsSynthesis,
+               uint32_t aLanguageOverride);
   // Features are composed of (1) features from style rules (2) features
   // from feature settings rules and (3) family-specific features.  (1) and
   // (3) are guaranteed to be mutually exclusive
@@ -198,6 +191,7 @@ struct gfxFontStyle {
   // Whether synthetic styles are allowed
   bool allowSyntheticWeight : 1;
   bool allowSyntheticStyle : 1;
+  bool allowSyntheticSmallCaps : 1;
 
   // some variant features require fallback which complicates the shaping
   // code, so set up a bool to indicate when shaping with fallback is needed
@@ -290,37 +284,27 @@ struct FontCacheSizes {
   size_t mShapedWords;    // memory used by the per-font shapedWord caches
 };
 
-class gfxFontCacheExpirationTracker
-    : public ExpirationTrackerImpl<gfxFont, 3, ::detail::PlaceholderLock,
-                                   ::detail::PlaceholderAutoLock> {
+class gfxFontCache final
+    : public ExpirationTrackerImpl<gfxFont, 3, mozilla::Mutex,
+                                   mozilla::MutexAutoLock> {
  protected:
-  typedef ::detail::PlaceholderLock Lock;
-  typedef ::detail::PlaceholderAutoLock AutoLock;
-
-  Lock mLock;
-
-  AutoLock FakeLock() { return AutoLock(mLock); }
-
-  Lock& GetMutex() override {
-    mozilla::AssertIsMainThreadOrServoFontMetricsLocked();
-    return mLock;
-  }
-
- public:
+  // Expiration tracker implementation.
   enum { FONT_TIMEOUT_SECONDS = 10 };
 
-  explicit gfxFontCacheExpirationTracker(nsIEventTarget* aEventTarget)
-      : ExpirationTrackerImpl<gfxFont, 3, Lock, AutoLock>(
-            FONT_TIMEOUT_SECONDS * 1000, "gfxFontCache", aEventTarget) {}
-};
+  typedef mozilla::Mutex Lock;
+  typedef mozilla::MutexAutoLock AutoLock;
+
+  // This protects the ExpirationTracker tables.
+  Lock mMutex = Lock("fontCacheExpirationMutex");
+
+  Lock& GetMutex() override { return mMutex; }
 
-class gfxFontCache final : private gfxFontCacheExpirationTracker {
  public:
-  enum { SHAPED_WORD_TIMEOUT_SECONDS = 60 };
-
   explicit gfxFontCache(nsIEventTarget* aEventTarget);
   ~gfxFontCache();
 
+  enum { SHAPED_WORD_TIMEOUT_SECONDS = 60 };
+
   /*
    * Get the global gfxFontCache.  You must call Init() before
    * calling this method --- the result will not be null.
@@ -351,23 +335,54 @@ class gfxFontCache final : private gfxFontCacheExpirationTracker {
   // Other gfxFont objects may be still in use but they will be pushed
   // into the expiration queues and removed.
   void Flush() {
-    mFonts.Clear();
+    {
+      mozilla::MutexAutoLock lock(mMutex);
+      mFonts.Clear();
+    }
     AgeAllGenerations();
   }
 
-  uint32_t Count() const { return mFonts.Count(); }
-
   void FlushShapedWordCaches();
   void NotifyGlyphsChanged();
 
+  void AgeCachedWords();
+
+  void RunWordCacheExpirationTimer() {
+    if (!mTimerRunning) {
+      mozilla::MutexAutoLock lock(mMutex);
+      if (!mTimerRunning && mWordCacheExpirationTimer) {
+        mWordCacheExpirationTimer->InitWithNamedFuncCallback(
+            WordCacheExpirationTimerCallback, this,
+            SHAPED_WORD_TIMEOUT_SECONDS * 1000, nsITimer::TYPE_REPEATING_SLACK,
+            "gfxFontCache::WordCacheExpiration");
+        mTimerRunning = true;
+      }
+    }
+  }
+  void PauseWordCacheExpirationTimer() {
+    if (mTimerRunning) {
+      mozilla::MutexAutoLock lock(mMutex);
+      if (mTimerRunning && mWordCacheExpirationTimer) {
+        mWordCacheExpirationTimer->Cancel();
+        mTimerRunning = false;
+      }
+    }
+  }
+
   void AddSizeOfExcludingThis(mozilla::MallocSizeOf aMallocSizeOf,
                               FontCacheSizes* aSizes) const;
   void AddSizeOfIncludingThis(mozilla::MallocSizeOf aMallocSizeOf,
                               FontCacheSizes* aSizes) const;
 
-  void AgeAllGenerations() { AgeAllGenerationsLocked(FakeLock()); }
+  void AgeAllGenerations() {
+    AutoLock lock(mMutex);
+    AgeAllGenerationsLocked(lock);
+  }
 
-  void RemoveObject(gfxFont* aFont) { RemoveObjectLocked(aFont, FakeLock()); }
+  void RemoveObject(gfxFont* aFont) {
+    AutoLock lock(mMutex);
+    RemoveObjectLocked(aFont, lock);
+  }
 
  protected:
   class MemoryReporter final : public nsIMemoryReporter {
@@ -388,18 +403,18 @@ class gfxFontCache final : private gfxFontCacheExpirationTracker {
   };
 
   nsresult AddObject(gfxFont* aFont) {
-    return AddObjectLocked(aFont, FakeLock());
+    AutoLock lock(mMutex);
+    return AddObjectLocked(aFont, lock);
   }
 
   // This gets called when the timeout has expired on a zero-refcount
   // font; we just delete it.
-  void NotifyExpiredLocked(gfxFont* aFont, const AutoLock&) override {
-    NotifyExpired(aFont);
-  }
-
+  void NotifyExpiredLocked(gfxFont* aFont, const AutoLock&)
+      REQUIRES(mMutex) override;
   void NotifyExpired(gfxFont* aFont);
 
   void DestroyFont(gfxFont* aFont);
+  void DestroyFontLocked(gfxFont* aFont) REQUIRES(mMutex);
 
   static gfxFontCache* gGlobalCache;
 
@@ -438,10 +453,12 @@ class gfxFontCache final : private gfxFontCacheExpirationTracker {
     gfxFont* MOZ_UNSAFE_REF("tracking for deferred deletion") mFont;
   };
 
-  nsTHashtable<HashEntry> mFonts;
+  nsTHashtable<HashEntry> mFonts GUARDED_BY(mMutex);
 
   static void WordCacheExpirationTimerCallback(nsITimer* aTimer, void* aCache);
-  nsCOMPtr<nsITimer> mWordCacheExpirationTimer;
+
+  nsCOMPtr<nsITimer> mWordCacheExpirationTimer GUARDED_BY(mMutex);
+  std::atomic<bool> mTimerRunning = false;
 };
 
 class gfxTextPerfMetrics {
@@ -498,6 +515,9 @@ class gfxTextPerfMetrics {
 
 namespace mozilla {
 namespace gfx {
+
+class UnscaledFont;
+
 // Flags that live in the gfxShapedText::mFlags field.
 // (Note that gfxTextRun has an additional mFlags2 field for use
 // by textrun clients like nsTextFrame.)
@@ -653,7 +673,7 @@ class gfxTextRunFactory {
 class gfxFontShaper {
  public:
   typedef mozilla::gfx::DrawTarget DrawTarget;
-  typedef mozilla::unicode::Script Script;
+  typedef mozilla::intl::Script Script;
 
   enum class RoundingFlags : uint8_t { kRoundX = 0x01, kRoundY = 0x02 };
 
@@ -712,7 +732,7 @@ MOZ_MAKE_ENUM_CLASS_BITWISE_OPERATORS(gfxFontShaper::RoundingFlags)
  */
 class gfxShapedText {
  public:
-  typedef mozilla::unicode::Script Script;
+  typedef mozilla::intl::Script Script;
 
   gfxShapedText(uint32_t aLength, mozilla::gfx::ShapedTextFlags aFlags,
                 uint16_t aAppUnitsPerDevUnit)
@@ -1242,7 +1262,7 @@ class gfxShapedText {
  */
 class gfxShapedWord final : public gfxShapedText {
  public:
-  typedef mozilla::unicode::Script Script;
+  typedef mozilla::intl::Script Script;
 
   // Create a ShapedWord that can hold glyphs for aLength characters,
   // with mCharacterGlyphs sized appropriately.
@@ -1386,7 +1406,8 @@ class gfxShapedWord final : public gfxShapedText {
 
   gfxFontShaper::RoundingFlags mRounding;
 
-  uint32_t mAgeCounter;
+  // With multithreaded shaping, this may be updated by any thread.
+  std::atomic<uint32_t> mAgeCounter;
 
   // The mCharGlyphsStorage array is actually a variable-size member;
   // when the ShapedWord is created, its size will be increased as necessary
@@ -1407,7 +1428,7 @@ class gfxFont {
 
  protected:
   using DrawTarget = mozilla::gfx::DrawTarget;
-  using Script = mozilla::unicode::Script;
+  using Script = mozilla::intl::Script;
   using SVGContextPaint = mozilla::SVGContextPaint;
 
   using RoundingFlags = gfxFontShaper::RoundingFlags;
@@ -1418,7 +1439,12 @@ class gfxFont {
 
   nsrefcnt AddRef(void) {
     MOZ_ASSERT(int32_t(mRefCnt) >= 0, "illegal refcnt");
-    if (mExpirationState.IsTracked()) {
+    nsExpirationState state;
+    {
+      mozilla::AutoReadLock lock(mLock);
+      state = mExpirationState;
+    }
+    if (state.IsTracked()) {
       gfxFontCache::GetCache()->RemoveObject(this);
     }
     ++mRefCnt;
@@ -1429,15 +1455,15 @@ class gfxFont {
     MOZ_ASSERT(0 != mRefCnt, "dup release");
     --mRefCnt;
     NS_LOG_RELEASE(this, mRefCnt, "gfxFont");
-    if (mRefCnt == 0) {
+    nsrefcnt rval = mRefCnt;
+    if (!rval) {
       NotifyReleased();
       // |this| may have been deleted.
-      return 0;
     }
-    return mRefCnt;
+    return rval;
   }
 
-  int32_t GetRefCount() { return mRefCnt; }
+  int32_t GetRefCount() { return int32_t(mRefCnt); }
 
   // options to specify the kind of AA to be used when creating a font
   typedef enum : uint8_t {
@@ -1448,7 +1474,7 @@ class gfxFont {
   } AntialiasOption;
 
  protected:
-  nsAutoRefCnt mRefCnt;
+  mozilla::ThreadSafeAutoRefCnt mRefCnt;
 
   void NotifyReleased() {
     gfxFontCache* cache = gfxFontCache::GetCache();
@@ -1501,8 +1527,7 @@ class gfxFont {
   const nsCString& GetName() const { return mFontEntry->Name(); }
   const gfxFontStyle* GetStyle() const { return &mStyle; }
 
-  virtual mozilla::UniquePtr<gfxFont> CopyWithAntialiasOption(
-      AntialiasOption anAAOption) {
+  virtual gfxFont* CopyWithAntialiasOption(AntialiasOption anAAOption) const {
     // platforms where this actually matters should override
     return nullptr;
   }
@@ -1526,15 +1551,17 @@ class gfxFont {
   }
 
   // check whether this is an sfnt we can potentially use with harfbuzz
-  bool FontCanSupportHarfBuzz() { return mFontEntry->HasCmapTable(); }
+  bool FontCanSupportHarfBuzz() const { return mFontEntry->HasCmapTable(); }
 
   // check whether this is an sfnt we can potentially use with Graphite
-  bool FontCanSupportGraphite() { return mFontEntry->HasGraphiteTables(); }
+  bool FontCanSupportGraphite() const {
+    return mFontEntry->HasGraphiteTables();
+  }
 
   // Whether this is a font that may be doing full-color rendering,
   // and therefore needs us to use a mask for text-shadow even when
   // we're not actually blurring.
-  bool AlwaysNeedsMaskForShadow() {
+  bool AlwaysNeedsMaskForShadow() const {
     return mFontEntry->TryGetColorGlyphs() || mFontEntry->TryGetSVGData(this) ||
            mFontEntry->HasFontTable(TRUETYPE_TAG('C', 'B', 'D', 'T')) ||
            mFontEntry->HasFontTable(TRUETYPE_TAG('s', 'b', 'i', 'x'));
@@ -1590,6 +1617,10 @@ class gfxFont {
   virtual bool ShouldHintMetrics() const { return true; }
   virtual bool ShouldRoundXOffset(cairo_t* aCairo) const { return true; }
 
+  // Return the font's owned harfbuzz shaper, creating and initializing it if
+  // necessary; returns null if shaper initialization has failed.
+  gfxHarfBuzzShaper* GetHarfBuzzShaper();
+
   // Font metrics
   struct Metrics {
     gfxFloat capHeight;
@@ -1612,12 +1643,15 @@ class gfxFont {
 
     gfxFloat aveCharWidth;
     gfxFloat spaceWidth;
-    gfxFloat zeroWidth;  // -1 if there was no zero glyph
+    gfxFloat zeroWidth;         // -1 if there was no zero glyph
+    gfxFloat ideographicWidth;  // -1 if kWaterIdeograph is not supported
 
     gfxFloat ZeroOrAveCharWidth() const {
       return zeroWidth >= 0 ? zeroWidth : aveCharWidth;
     }
   };
+  // Unicode character used as basis for 'ic' unit:
+  static constexpr uint32_t kWaterIdeograph = 0x6C34;
 
   typedef nsFontMetrics::FontOrientation Orientation;
 
@@ -1626,7 +1660,7 @@ class gfxFont {
       return GetHorizontalMetrics();
     }
     if (!mVerticalMetrics) {
-      mVerticalMetrics = CreateVerticalMetrics();
+      CreateVerticalMetrics();
     }
     return *mVerticalMetrics;
   }
@@ -1752,16 +1786,16 @@ class gfxFont {
   nsExpirationState* GetExpirationState() { return &mExpirationState; }
 
   // Get the glyphID of a space
-  uint16_t GetSpaceGlyph() { return mSpaceGlyph; }
+  uint16_t GetSpaceGlyph() const { return mSpaceGlyph; }
 
   gfxGlyphExtents* GetOrCreateGlyphExtents(int32_t aAppUnitsPerDevUnit);
 
   void SetupGlyphExtents(DrawTarget* aDrawTarget, uint32_t aGlyphID,
                          bool aNeedTight, gfxGlyphExtents* aExtents);
 
-  virtual bool AllowSubpixelAA() { return true; }
+  virtual bool AllowSubpixelAA() const { return true; }
 
-  bool IsSyntheticBold() const { return mApplySyntheticBold; }
+  bool ApplySyntheticBold() const { return mApplySyntheticBold; }
 
   float AngleForSyntheticOblique() const;
   float SkewForSyntheticOblique() const;
@@ -1770,7 +1804,7 @@ class gfxFont {
   // For size S up to a threshold size T, we use (0.25 + 3S / 4T),
   // so that the result ranges from 0.25 to 1.0; thereafter,
   // simply use (S / T).
-  gfxFloat GetSyntheticBoldOffset() {
+  gfxFloat GetSyntheticBoldOffset() const {
     gfxFloat size = GetAdjustedSize();
     const gfxFloat threshold = 48.0;
     return size < threshold ? (0.25 + 0.75 * size / threshold)
@@ -1778,7 +1812,7 @@ class gfxFont {
   }
 
   gfxFontEntry* GetFontEntry() const { return mFontEntry.get(); }
-  bool HasCharacter(uint32_t ch) {
+  bool HasCharacter(uint32_t ch) const {
     if (!mIsValid || (mUnicodeRangeMap && !mUnicodeRangeMap->test(ch))) {
       return false;
     }
@@ -1793,7 +1827,7 @@ class gfxFont {
     mUnicodeRangeMap = aUnicodeRangeMap;
   }
 
-  uint16_t GetUVSGlyph(uint32_t aCh, uint32_t aVS) {
+  uint16_t GetUVSGlyph(uint32_t aCh, uint32_t aVS) const {
     if (!mIsValid) {
       return 0;
     }
@@ -1801,7 +1835,8 @@ class gfxFont {
   }
 
   template <typename T>
-  bool InitFakeSmallCapsRun(DrawTarget* aDrawTarget, gfxTextRun* aTextRun,
+  bool InitFakeSmallCapsRun(nsPresContext* aPresContext,
+                            DrawTarget* aDrawTarget, gfxTextRun* aTextRun,
                             const T* aText, uint32_t aOffset, uint32_t aLength,
                             FontMatchType aMatchType,
                             mozilla::gfx::ShapedTextFlags aOrientation,
@@ -1829,27 +1864,25 @@ class gfxFont {
                                RoundingFlags aRounding,
                                gfxTextPerfMetrics* aTextPerf);
 
-  // Ensure the ShapedWord cache is initialized. This MUST be called before
-  // any attempt to use GetShapedWord().
-  void InitWordCache() {
-    if (!mWordCache) {
-      mWordCache = mozilla::MakeUnique<nsTHashtable<CacheHashEntry>>();
-    }
-  }
-
   // Called by the gfxFontCache timer to increment the age of all the words,
-  // so that they'll expire after a sufficient period of non-use
-  void AgeCachedWords();
+  // so that they'll expire after a sufficient period of non-use.
+  // Returns true if the cache is now empty, otherwise false.
+  bool AgeCachedWords();
 
   // Discard all cached word records; called on memory-pressure notification.
   void ClearCachedWords() {
+    mozilla::AutoWriteLock lock(mLock);
     if (mWordCache) {
-      mWordCache->Clear();
+      ClearCachedWordsLocked();
     }
   }
+  void ClearCachedWordsLocked() REQUIRES(mLock) {
+    MOZ_ASSERT(mWordCache);
+    mWordCache->Clear();
+  }
 
   // Glyph rendering/geometry has changed, so invalidate data as necessary.
-  void NotifyGlyphsChanged();
+  void NotifyGlyphsChanged() const;
 
   virtual void AddSizeOfExcludingThis(mozilla::MallocSizeOf aMallocSizeOf,
                                       FontCacheSizes* aSizes) const;
@@ -1873,11 +1906,17 @@ class gfxFont {
   }
 
   virtual already_AddRefed<mozilla::gfx::ScaledFont> GetScaledFont(
-      DrawTarget* aTarget) = 0;
+      const TextRunDrawParams& aRunParams) = 0;
+  already_AddRefed<mozilla::gfx::ScaledFont> GetScaledFont(
+      mozilla::gfx::DrawTarget* aDrawTarget);
 
-  void InitializeScaledFont();
+  // gfxFont implementations may cache ScaledFont versions other than the
+  // default, so InitializeScaledFont must support explicitly specifying
+  // which ScaledFonts to initialize.
+  void InitializeScaledFont(
+      const RefPtr<mozilla::gfx::ScaledFont>& aScaledFont);
 
-  bool KerningDisabled() { return mKerningSet && !mKerningEnabled; }
+  bool KerningDisabled() const { return mKerningSet && !mKerningEnabled; }
 
   /**
    * Subclass this object to be notified of glyph changes. Delete the object
@@ -1904,7 +1943,7 @@ class gfxFont {
   };
   friend class GlyphChangeObserver;
 
-  bool GlyphsMayChange() {
+  bool GlyphsMayChange() const {
     // Currently only fonts with SVG glyphs can have animated glyphs
     return mFontEntry->TryGetSVGData(this);
   }
@@ -1918,23 +1957,23 @@ class gfxFont {
   // If (and ONLY if) TryGetMathTable() has returned true, the MathTable()
   // method may be called to access the gfxMathTable data.
   bool TryGetMathTable();
-  gfxMathTable* MathTable() {
+  gfxMathTable* MathTable() const {
     MOZ_RELEASE_ASSERT(mMathTable,
                        "A successful call to TryGetMathTable() must be "
                        "performed before calling this function");
-    return mMathTable.get();
+    return mMathTable;
   }
 
   // Return a cloned font resized and offset to simulate sub/superscript
   // glyphs. This does not add a reference to the returned font.
-  gfxFont* GetSubSuperscriptFont(int32_t aAppUnitsPerDevPixel);
+  gfxFont* GetSubSuperscriptFont(int32_t aAppUnitsPerDevPixel) const;
 
   bool HasColorGlyphFor(uint32_t aCh, uint32_t aNextCh);
 
  protected:
-  virtual const Metrics& GetHorizontalMetrics() = 0;
+  virtual const Metrics& GetHorizontalMetrics() const = 0;
 
-  mozilla::UniquePtr<const Metrics> CreateVerticalMetrics();
+  void CreateVerticalMetrics();
 
   // Template parameters for DrawGlyphs/DrawOneGlyph, used to select
   // simplified versions of the methods in the most common cases.
@@ -1978,7 +2017,7 @@ class gfxFont {
   // Return a font that is a "clone" of this one, but reduced to 80% size
   // (and with variantCaps set to normal). This does not add a reference to
   // the returned font.
-  gfxFont* GetSmallCapsFont();
+  gfxFont* GetSmallCapsFont() const;
 
   // subclasses may provide (possibly hinted) glyph widths (in font units);
   // if they do not override this, harfbuzz will use unhinted widths
@@ -1990,7 +2029,7 @@ class gfxFont {
   virtual int32_t GetGlyphWidth(uint16_t aGID) { return -1; }
 
   virtual bool GetGlyphBounds(uint16_t aGID, gfxRect* aBounds,
-                              bool aTight = false) {
+                              bool aTight = false) const {
     return false;
   }
 
@@ -2001,13 +2040,13 @@ class gfxFont {
   void RemoveGlyphChangeObserver(GlyphChangeObserver* aObserver);
 
   // whether font contains substitution lookups containing spaces
-  bool HasSubstitutionRulesWithSpaceLookups(Script aRunScript);
+  bool HasSubstitutionRulesWithSpaceLookups(Script aRunScript) const;
 
   // do spaces participate in shaping rules? if so, can't used word cache
   // Note that this function uses HasGraphiteSpaceContextuals, so it can only
   // return a "hint" to the correct answer. The  calling code must ensure it
   // performs safe actions independent of the value returned.
-  tainted_boolean_hint SpaceMayParticipateInShaping(Script aRunScript);
+  tainted_boolean_hint SpaceMayParticipateInShaping(Script aRunScript) const;
 
   // For 8-bit text, expand to 16-bit and then call the following method.
   bool ShapeText(DrawTarget* aContext, const uint8_t* aText,
@@ -2057,7 +2096,7 @@ class gfxFont {
                                      bool aVertical, RoundingFlags aRounding,
                                      gfxTextRun* aTextRun);
 
-  void CheckForFeaturesInvolvingSpace();
+  void CheckForFeaturesInvolvingSpace() const;
 
   // whether a given feature is included in feature settings from both the
   // font and the style. aFeatureOn set if resolved feature value is non-zero
@@ -2068,6 +2107,7 @@ class gfxFont {
   static nsTHashSet<uint32_t>* sDefaultFeatures;
 
   RefPtr<gfxFontEntry> mFontEntry;
+  mutable mozilla::RWLock mLock;
 
   struct CacheHashKey {
     union {
@@ -2154,35 +2194,42 @@ class gfxFont {
     mozilla::UniquePtr<gfxShapedWord> mShapedWord;
   };
 
-  mozilla::UniquePtr<nsTHashtable<CacheHashEntry>> mWordCache;
+  mozilla::UniquePtr<nsTHashtable<CacheHashEntry>> mWordCache GUARDED_BY(mLock);
 
   static const uint32_t kShapedWordCacheMaxAge = 3;
 
-  nsTArray<mozilla::UniquePtr<gfxGlyphExtents>> mGlyphExtentsArray;
-  mozilla::UniquePtr<nsTHashSet<GlyphChangeObserver*>> mGlyphChangeObservers;
+  nsTArray<mozilla::UniquePtr<gfxGlyphExtents>> mGlyphExtentsArray
+      GUARDED_BY(mLock);
+  mozilla::UniquePtr<nsTHashSet<GlyphChangeObserver*>> mGlyphChangeObservers
+      GUARDED_BY(mLock);
 
   // a copy of the font without antialiasing, if needed for separate
   // measurement by mathml code
-  mozilla::UniquePtr<gfxFont> mNonAAFont;
+  mozilla::Atomic<gfxFont*> mNonAAFont;
 
   // we create either or both of these shapers when needed, depending
   // whether the font has graphite tables, and whether graphite shaping
   // is actually enabled
-  mozilla::UniquePtr<gfxFontShaper> mHarfBuzzShaper;
-  mozilla::UniquePtr<gfxFontShaper> mGraphiteShaper;
+  mozilla::Atomic<gfxHarfBuzzShaper*> mHarfBuzzShaper;
+  mozilla::Atomic<gfxGraphiteShaper*> mGraphiteShaper;
 
-  // if a userfont with unicode-range specified, contains map of *possible*
-  // ranges supported by font
+  // If a userfont with unicode-range specified, contains map of *possible*
+  // ranges supported by font. This is set during user-font initialization,
+  // before the font is available to other threads, and thereafter is inert
+  // so no guard is needed.
   RefPtr<gfxCharacterMap> mUnicodeRangeMap;
 
+  // This is immutable once initialized by the constructor, so does not need
+  // locking.
   RefPtr<mozilla::gfx::UnscaledFont> mUnscaledFont;
-  RefPtr<mozilla::gfx::ScaledFont> mAzureScaledFont;
+
+  mozilla::Atomic<mozilla::gfx::ScaledFont*> mAzureScaledFont;
 
   // For vertical metrics, created on demand.
-  mozilla::UniquePtr<const Metrics> mVerticalMetrics;
+  mozilla::Atomic<Metrics*> mVerticalMetrics;
 
   // Table used for MathML layout.
-  mozilla::UniquePtr<gfxMathTable> mMathTable;
+  mozilla::Atomic<gfxMathTable*> mMathTable;
 
   gfxFontStyle mStyle;
   mutable gfxFloat mAdjustedSize;
@@ -2192,7 +2239,7 @@ class gfxFont {
   // This is OK because we only multiply by this factor, never divide.
   float mFUnitsConvFactor;
 
-  nsExpirationState mExpirationState;
+  nsExpirationState mExpirationState GUARDED_BY(mLock);
 
   // Glyph ID of the font's <space> glyph, zero if missing
   uint16_t mSpaceGlyph = 0;
@@ -2209,7 +2256,7 @@ class gfxFont {
   bool mKerningSet;      // kerning explicitly set?
   bool mKerningEnabled;  // if set, on or off?
 
-  bool mMathInitialized;  // TryGetMathTable() called?
+  mozilla::Atomic<bool> mMathInitialized;  // TryGetMathTable() called?
 
   // Helper for subclasses that want to initialize standard metrics from the
   // tables of sfnt (TrueType/OpenType) fonts.
@@ -2266,20 +2313,21 @@ class gfxFont {
 
 struct MOZ_STACK_CLASS TextRunDrawParams {
   RefPtr<mozilla::gfx::DrawTarget> dt;
-  gfxContext* context;
-  gfxFont::Spacing* spacing;
-  gfxTextRunDrawCallbacks* callbacks;
-  mozilla::SVGContextPaint* runContextPaint;
-  mozilla::gfx::Float direction;
-  double devPerApp;
-  nscolor textStrokeColor;
-  gfxPattern* textStrokePattern;
-  const mozilla::gfx::StrokeOptions* strokeOpts;
-  const mozilla::gfx::DrawOptions* drawOpts;
-  DrawMode drawMode;
-  bool isVerticalRun;
-  bool isRTL;
-  bool paintSVGGlyphs;
+  gfxContext* context = nullptr;
+  gfxFont::Spacing* spacing = nullptr;
+  gfxTextRunDrawCallbacks* callbacks = nullptr;
+  mozilla::SVGContextPaint* runContextPaint = nullptr;
+  mozilla::gfx::Float direction = 1.0f;
+  double devPerApp = 1.0;
+  nscolor textStrokeColor = 0;
+  gfxPattern* textStrokePattern = nullptr;
+  const mozilla::gfx::StrokeOptions* strokeOpts = nullptr;
+  const mozilla::gfx::DrawOptions* drawOpts = nullptr;
+  DrawMode drawMode = DrawMode::GLYPH_FILL;
+  bool isVerticalRun = false;
+  bool isRTL = false;
+  bool paintSVGGlyphs = true;
+  bool allowGDI = true;
 };
 
 struct MOZ_STACK_CLASS FontDrawParams {
