@@ -19,17 +19,19 @@
 #include "gfxUserFontSet.h"
 #include "mozilla/MemoryReporting.h"
 #include "mozilla/RefPtr.h"
+#include "mozilla/intl/UnicodeScriptCodes.h"
 #include "nsPoint.h"
 #include "nsString.h"
 #include "nsTArray.h"
+#include "nsTHashSet.h"
 #include "nsTextFrameUtils.h"
 #include "DrawMode.h"
 #include "harfbuzz/hb.h"
-#include "nsUnicodeScriptCodes.h"
 #include "nsColor.h"
+#include "nsFrameList.h"
 #include "X11UndefineNone.h"
 
-#ifdef DEBUG
+#ifdef DEBUG_FRAME_DUMP
 #  include <stdio.h>
 #endif
 
@@ -187,6 +189,10 @@ class gfxTextRun : public gfxShapedText {
     AutoWithoutManualInSameWord
   };
 
+  static bool IsOptionalHyphenBreak(HyphenType aType) {
+    return aType >= HyphenType::Soft;
+  }
+
   struct HyphenationState {
     uint32_t mostRecentBoundary = 0;
     bool hasManualHyphen = false;
@@ -220,6 +226,9 @@ class gfxTextRun : public gfxShapedText {
     // be constant for a given textrun.
     virtual gfxFloat GetHyphenWidth() const = 0;
 
+    // Return orientation flags to be used when creating a hyphen textrun.
+    virtual mozilla::gfx::ShapedTextFlags GetShapedTextFlags() const = 0;
+
     typedef gfxFont::Spacing Spacing;
 
     /**
@@ -251,6 +260,7 @@ class gfxTextRun : public gfxShapedText {
     gfxFloat* advanceWidth = nullptr;
     mozilla::SVGContextPaint* contextPaint = nullptr;
     gfxTextRunDrawCallbacks* callbacks = nullptr;
+    bool allowGDI = true;
     explicit DrawParams(gfxContext* aContext) : context(aContext) {}
   };
 
@@ -302,6 +312,12 @@ class gfxTextRun : public gfxShapedText {
                        aDrawTargetForTightBoundingBox, aProvider);
   }
 
+  void GetLineHeightMetrics(Range aRange, gfxFloat& aAscent,
+                            gfxFloat& aDescent) const;
+  void GetLineHeightMetrics(gfxFloat& aAscent, gfxFloat& aDescent) const {
+    GetLineHeightMetrics(Range(this), aAscent, aDescent);
+  }
+
   /**
    * Computes just the advance width for a substring.
    * Uses GetSpacing from aBreakProvider.
@@ -756,8 +772,8 @@ class gfxTextRun : public gfxShapedText {
     return advance;
   }
 
-#ifdef DEBUG
-  void Dump(FILE* aOutput);
+#ifdef DEBUG_FRAME_DUMP
+  void Dump(FILE* aOutput = stderr);
 #endif
 
  protected:
@@ -818,7 +834,8 @@ class gfxTextRun : public gfxShapedText {
   // Advance aRange.start to the start of the nearest ligature, back
   // up aRange.end to the nearest ligature end; may result in
   // aRange->start == aRange->end.
-  void ShrinkToLigatureBoundaries(Range* aRange) const;
+  // Returns whether any adjustment was made.
+  bool ShrinkToLigatureBoundaries(Range* aRange) const;
   // result in appunits
   gfxFloat GetPartialLigatureWidth(Range aRange,
                                    PropertyProvider* aProvider) const;
@@ -877,60 +894,33 @@ class gfxTextRun : public gfxShapedText {
   nsTextFrameUtils::Flags
       mFlags2;  // additional flags (see also gfxShapedText::mFlags)
 
-  bool mSkipDrawing;  // true if the font group we used had a user font
-                      // download that's in progress, so we should hide text
-                      // until the download completes (or timeout fires)
-  bool mReleasedFontGroup;  // we already called NS_RELEASE on
-                            // mFontGroup, so don't do it again
-  bool mHasGlyphRunArray;   // whether we're using an array or
-                            // just storing a single glyphrun
+  bool mDontSkipDrawing;  // true if the text run must not skip drawing, even if
+                          // waiting for a user font download, e.g. because we
+                          // are using it to draw canvas text
+  bool mReleasedFontGroup;                // we already called NS_RELEASE on
+                                          // mFontGroup, so don't do it again
+  bool mReleasedFontGroupSkippedDrawing;  // whether our old mFontGroup value
+                                          // was set to skip drawing
+  bool mHasGlyphRunArray;                 // whether we're using an array or
+                                          // just storing a single glyphrun
 
   // shaping state for handling variant fallback features
   // such as subscript/superscript variant glyphs
   ShapingState mShapingState;
 };
 
-enum class FallbackTypes : uint8_t {
-  // Font fallback used a font configured in Preferences
-  FallbackToPrefsFont = 1 << 0,
-  // Font fallback used a font with FontVisibility::Base
-  FallbackToBaseFont = 1 << 1,
-  // Font fallback used a font with FontVisibility::LangPack
-  FallbackToLangPackFont = 1 << 2,
-  // Font fallback used a font with FontVisibility::User
-  FallbackToUserFont = 1 << 3,
-  // Rendered missing-glyph because no font available for the character
-  MissingFont = 1 << 4,
-  // Rendered missing-glyph but a LangPack font could have been used
-  MissingFontLangPack = 1 << 5,
-  // Rendered missing-glyph but a User font could have been used
-  MissingFontUser = 1 << 6,
-};
-
-MOZ_MAKE_ENUM_CLASS_BITWISE_OPERATORS(FallbackTypes)
-
-struct FontMatchingStats {
-  // Set of names that have been looked up (whether successfully or not).
-  nsTHashtable<nsCStringHashKey> mFamilyNames;
-  // Number of font-family names resolved at each level of visibility.
-  uint32_t mBaseFonts = 0;
-  uint32_t mLangPackFonts = 0;
-  uint32_t mUserFonts = 0;
-  uint32_t mWebFonts = 0;
-  FallbackTypes mFallbacks = FallbackTypes(0);
-};
-
 class gfxFontGroup final : public gfxTextRunFactory {
  public:
-  typedef mozilla::unicode::Script Script;
+  typedef mozilla::intl::Script Script;
   typedef gfxShapedText::CompressedGlyph CompressedGlyph;
 
   static void
   Shutdown();  // platform must call this to release the languageAtomService
 
-  gfxFontGroup(const mozilla::FontFamilyList& aFontFamilyList,
-               const gfxFontStyle* aStyle, gfxTextPerfMetrics* aTextPerf,
-               FontMatchingStats* aFontMatchingStats,
+  gfxFontGroup(nsPresContext* aPresContext,
+               const mozilla::StyleFontFamilyList& aFontFamilyList,
+               const gfxFontStyle* aStyle, nsAtom* aLanguage,
+               bool aExplicitLanguage, gfxTextPerfMetrics* aTextPerf,
                gfxUserFontSet* aUserFontSet, gfxFloat aDevToCssSize);
 
   virtual ~gfxFontGroup();
@@ -951,6 +941,10 @@ class gfxFontGroup final : public gfxTextRunFactory {
 
   const gfxFontStyle* GetStyle() const { return &mStyle; }
 
+  // Get the presContext for which this fontGroup was constructed. This may be
+  // null! (In the case of canvas not connected to a document.)
+  nsPresContext* GetPresContext() const { return mPresContext; }
+
   /**
    * The listed characters should be treated as invisible and zero-width
    * when creating textruns.
@@ -1009,8 +1003,9 @@ class gfxFontGroup final : public gfxTextRunFactory {
    * The caller is responsible for deleting the returned text run
    * when no longer required.
    */
-  already_AddRefed<gfxTextRun> MakeHyphenTextRun(DrawTarget* aDrawTarget,
-                                                 uint32_t aAppUnitsPerDevUnit);
+  already_AddRefed<gfxTextRun> MakeHyphenTextRun(
+      DrawTarget* aDrawTarget, mozilla::gfx::ShapedTextFlags aFlags,
+      uint32_t aAppUnitsPerDevUnit);
 
   /**
    * Check whether a given font (specified by its gfxFontEntry)
@@ -1044,7 +1039,7 @@ class gfxFontGroup final : public gfxTextRunFactory {
   uint64_t GetRebuildGeneration();
 
   // used when logging text performance
-  gfxTextPerfMetrics* GetTextPerfMetrics() { return mTextPerf; }
+  gfxTextPerfMetrics* GetTextPerfMetrics() const { return mTextPerf; }
 
   // This will call UpdateUserFonts() if the user font set is changed.
   void SetUserFontSet(gfxUserFontSet* aUserFontSet);
@@ -1084,11 +1079,14 @@ class gfxFontGroup final : public gfxTextRunFactory {
       // Forget cached fonts that may no longer be valid.
       mLastPrefFamily = FontFamily();
       mLastPrefFont = nullptr;
+      mDefaultFont = nullptr;
       mFonts.Clear();
       BuildFontList();
     }
   }
 
+  nsAtom* Language() const { return mLanguage.get(); }
+
  protected:
   friend class mozilla::PostTraversalTask;
 
@@ -1110,10 +1108,12 @@ class gfxFontGroup final : public gfxTextRunFactory {
 
   // search through pref fonts for a character, return nullptr if no matching
   // pref font
-  gfxFont* WhichPrefFontSupportsChar(uint32_t aCh, uint32_t aNextCh);
+  gfxFont* WhichPrefFontSupportsChar(uint32_t aCh, uint32_t aNextCh,
+                                     eFontPresentation aPresentation);
 
   gfxFont* WhichSystemFontSupportsChar(uint32_t aCh, uint32_t aNextCh,
-                                       Script aRunScript);
+                                       Script aRunScript,
+                                       eFontPresentation aPresentation);
 
   template <typename T>
   void ComputeRanges(nsTArray<TextRange>& aRanges, const T* aString,
@@ -1300,7 +1300,8 @@ class gfxFontGroup final : public gfxTextRunFactory {
 
     bool IsSharedFamily() const { return mIsSharedFamily; }
     bool IsUserFontContainer() const {
-      return FontEntry()->mIsUserFontContainer;
+      gfxFontEntry* fe = FontEntry();
+      return fe && fe->mIsUserFontContainer;
     }
     bool IsLoading() const { return mLoading; }
     bool IsInvalid() const { return mInvalid; }
@@ -1317,8 +1318,8 @@ class gfxFontGroup final : public gfxTextRunFactory {
         return false;
       }
       MOZ_ASSERT(IsUserFontContainer());
-      return static_cast<gfxUserFontEntry*>(FontEntry())
-          ->CharacterInUnicodeRange(aCh);
+      auto* ufe = static_cast<gfxUserFontEntry*>(FontEntry());
+      return ufe && ufe->CharacterInUnicodeRange(aCh);
     }
 
     void SetFont(gfxFont* aFont) {
@@ -1359,9 +1360,11 @@ class gfxFontGroup final : public gfxTextRunFactory {
     bool mHasFontEntry : 1;
   };
 
+  nsPresContext* mPresContext = nullptr;
+
   // List of font families, either named or generic.
   // Generic names map to system pref fonts based on language.
-  mozilla::FontFamilyList mFamilyList;
+  mozilla::StyleFontFamilyList mFamilyList;
 
   // Fontlist containing a font entry for each family found. gfxFont objects
   // are created as needed and userfont loads are initiated when needed.
@@ -1371,6 +1374,8 @@ class gfxFontGroup final : public gfxTextRunFactory {
   RefPtr<gfxFont> mDefaultFont;
   gfxFontStyle mStyle;
 
+  RefPtr<nsAtom> mLanguage;
+
   gfxFloat mUnderlineOffset;
   gfxFloat mHyphenWidth;
   gfxFloat mDevToCssSize;
@@ -1381,8 +1386,6 @@ class gfxFontGroup final : public gfxTextRunFactory {
 
   gfxTextPerfMetrics* mTextPerf;
 
-  FontMatchingStats* mFontMatchingStats;
-
   // Cache a textrun representing an ellipsis (useful for CSS text-overflow)
   // at a specific appUnitsPerDevPixel size and orientation
   RefPtr<gfxTextRun> mCachedEllipsisTextRun;
@@ -1399,6 +1402,12 @@ class gfxFontGroup final : public gfxTextRunFactory {
                       // download to complete (or fallback
                       // timer to fire)
 
+  bool mExplicitLanguage;  // Does mLanguage come from an explicit attribute?
+
+  // Generic font family used to select among font prefs during fallback.
+  mozilla::StyleGenericFontFamily mFallbackGeneric =
+      mozilla::StyleGenericFontFamily::None;
+
   uint32_t mFontListGeneration = 0;  // platform font list generation for this
                                      // fontgroup
 
@@ -1464,12 +1473,17 @@ class gfxFontGroup final : public gfxTextRunFactory {
   // Helper for font-matching:
   // search all faces in a family for a fallback in cases where it's unclear
   // whether the family might have a font for a given character
-  gfxFont* FindFallbackFaceForChar(const FamilyFace& aFamily, uint32_t aCh);
+  gfxFont* FindFallbackFaceForChar(const FamilyFace& aFamily, uint32_t aCh,
+                                   uint32_t aNextCh,
+                                   eFontPresentation aPresentation);
 
   gfxFont* FindFallbackFaceForChar(mozilla::fontlist::Family* aFamily,
-                                   uint32_t aCh);
+                                   uint32_t aCh, uint32_t aNextCh,
+                                   eFontPresentation aPresentation);
 
-  gfxFont* FindFallbackFaceForChar(gfxFontFamily* aFamily, uint32_t aCh);
+  gfxFont* FindFallbackFaceForChar(gfxFontFamily* aFamily, uint32_t aCh,
+                                   uint32_t aNextCh,
+                                   eFontPresentation aPresentation);
 
   // helper methods for looking up fonts
 
@@ -1509,7 +1523,7 @@ class gfxMissingFontRecorder {
   }
 
   // record this script code in our mMissingFonts bitset
-  void RecordScript(mozilla::unicode::Script aScriptCode) {
+  void RecordScript(mozilla::intl::Script aScriptCode) {
     mMissingFonts[static_cast<uint32_t>(aScriptCode) >> 5] |=
         (1 << (static_cast<uint32_t>(aScriptCode) & 0x1f));
   }
@@ -1525,8 +1539,7 @@ class gfxMissingFontRecorder {
  private:
   // Number of 32-bit words needed for the missing-script flags
   static const uint32_t kNumScriptBitsWords =
-      ((static_cast<int>(mozilla::unicode::Script::NUM_SCRIPT_CODES) + 31) /
-       32);
+      ((static_cast<int>(mozilla::intl::Script::NUM_SCRIPT_CODES) + 31) / 32);
   uint32_t mMissingFonts[kNumScriptBitsWords];
 };
 
