@@ -2,48 +2,45 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
-extern crate env_logger;
-extern crate euclid;
-
 use gleam::gl;
-use glutin::{self, GlContext};
+use glutin;
 use std::env;
 use std::path::PathBuf;
 use webrender;
 use winit;
+use winit::platform::run_return::EventLoopExtRunReturn;
 use webrender::{DebugFlags, ShaderPrecacheFlags};
 use webrender::api::*;
+use webrender::render_api::*;
 use webrender::api::units::*;
 
-
 struct Notifier {
-    events_proxy: winit::EventsLoopProxy,
+    events_proxy: winit::event_loop::EventLoopProxy<()>,
 }
 
 impl Notifier {
-    fn new(events_proxy: winit::EventsLoopProxy) -> Notifier {
+    fn new(events_proxy: winit::event_loop::EventLoopProxy<()>) -> Notifier {
         Notifier { events_proxy }
     }
 }
 
 impl RenderNotifier for Notifier {
-    fn clone(&self) -> Box<RenderNotifier> {
+    fn clone(&self) -> Box<dyn RenderNotifier> {
         Box::new(Notifier {
             events_proxy: self.events_proxy.clone(),
         })
     }
 
-    fn wake_up(&self) {
+    fn wake_up(&self, _composite_needed: bool) {
         #[cfg(not(target_os = "android"))]
-        let _ = self.events_proxy.wakeup();
+        let _ = self.events_proxy.send_event(());
     }
 
     fn new_frame_ready(&self,
                        _: DocumentId,
                        _scrolled: bool,
-                       _composite_needed: bool,
-                       _render_time: Option<u64>) {
-        self.wake_up();
+                       composite_needed: bool) {
+        self.wake_up(composite_needed);
     }
 }
 
@@ -55,14 +52,14 @@ pub trait HandyDandyRectBuilder {
 // values to build a f32 LayoutRect
 impl HandyDandyRectBuilder for (i32, i32) {
     fn to(&self, x2: i32, y2: i32) -> LayoutRect {
-        LayoutRect::new(
+        LayoutRect::from_origin_and_size(
             LayoutPoint::new(self.0 as f32, self.1 as f32),
             LayoutSize::new((x2 - self.0) as f32, (y2 - self.1) as f32),
         )
     }
 
     fn by(&self, w: i32, h: i32) -> LayoutRect {
-        LayoutRect::new(
+        LayoutRect::from_origin_and_size(
             LayoutPoint::new(self.0 as f32, self.1 as f32),
             LayoutSize::new(w as f32, h as f32),
         )
@@ -77,7 +74,7 @@ pub trait Example {
 
     fn render(
         &mut self,
-        api: &RenderApi,
+        api: &mut RenderApi,
         builder: &mut DisplayListBuilder,
         txn: &mut Transaction,
         device_size: DeviceIntSize,
@@ -86,20 +83,20 @@ pub trait Example {
     );
     fn on_event(
         &mut self,
-        _: winit::WindowEvent,
-        _: &RenderApi,
+        _: winit::event::WindowEvent,
+        _: &winit::window::Window,
+        _: &mut RenderApi,
         _: DocumentId,
     ) -> bool {
         false
     }
-    fn get_image_handlers(
+    fn get_image_handler(
         &mut self,
-        _gl: &gl::Gl,
-    ) -> (Option<Box<webrender::ExternalImageHandler>>,
-          Option<Box<webrender::OutputImageHandler>>) {
-        (None, None)
+        _gl: &dyn gl::Gl,
+    ) -> Option<Box<dyn ExternalImageHandler>> {
+        None
     }
-    fn draw_custom(&mut self, _gl: &gl::Gl) {
+    fn draw_custom(&mut self, _gl: &dyn gl::Gl) {
     }
 }
 
@@ -109,6 +106,17 @@ pub fn main_wrapper<E: Example>(
 ) {
     env_logger::init();
 
+    #[cfg(target_os = "macos")]
+    {
+        use core_foundation::{self as cf, base::TCFType};
+        let i = cf::bundle::CFBundle::main_bundle().info_dictionary();
+        let mut i = unsafe { i.to_mutable() };
+        i.set(
+            cf::string::CFString::new("NSSupportsAutomaticGraphicsSwitching"),
+            cf::boolean::CFBoolean::true_value().into_CFType(),
+        );
+    }
+
     let args: Vec<String> = env::args().collect();
     let res_path = if args.len() > 1 {
         Some(PathBuf::from(&args[1]))
@@ -116,36 +124,37 @@ pub fn main_wrapper<E: Example>(
         None
     };
 
-    let mut events_loop = winit::EventsLoop::new();
-    let context_builder = glutin::ContextBuilder::new()
+    let mut events_loop = winit::event_loop::EventLoop::new();
+    let window_builder = winit::window::WindowBuilder::new()
+        .with_title(E::TITLE)
+        .with_inner_size(winit::dpi::LogicalSize::new(E::WIDTH as f64, E::HEIGHT as f64));
+    let windowed_context = glutin::ContextBuilder::new()
         .with_gl(glutin::GlRequest::GlThenGles {
             opengl_version: (3, 2),
             opengles_version: (3, 0),
-        });
-    let window_builder = winit::WindowBuilder::new()
-        .with_title(E::TITLE)
-        .with_multitouch()
-        .with_dimensions(winit::dpi::LogicalSize::new(E::WIDTH as f64, E::HEIGHT as f64));
-    let window = glutin::GlWindow::new(window_builder, context_builder, &events_loop)
+        })
+        .build_windowed(window_builder, &events_loop)
         .unwrap();
 
-    unsafe {
-        window.make_current().ok();
-    }
+    let windowed_context = unsafe { windowed_context.make_current().unwrap() };
 
-    let gl = match window.get_api() {
+    let gl = match windowed_context.get_api() {
         glutin::Api::OpenGl => unsafe {
-            gl::GlFns::load_with(|symbol| window.get_proc_address(symbol) as *const _)
+            gl::GlFns::load_with(
+                |symbol| windowed_context.get_proc_address(symbol) as *const _
+            )
         },
         glutin::Api::OpenGlEs => unsafe {
-            gl::GlesFns::load_with(|symbol| window.get_proc_address(symbol) as *const _)
+            gl::GlesFns::load_with(
+                |symbol| windowed_context.get_proc_address(symbol) as *const _
+            )
         },
         glutin::Api::WebGl => unimplemented!(),
     };
 
     println!("OpenGL version {}", gl.get_string(gl::VERSION));
     println!("Shader resource path: {:?}", res_path);
-    let device_pixel_ratio = window.get_hidpi_factor() as f32;
+    let device_pixel_ratio = windowed_context.window().scale_factor() as f32;
     println!("Device pixel ratio: {}", device_pixel_ratio);
 
     println!("Loading shaders...");
@@ -153,18 +162,16 @@ pub fn main_wrapper<E: Example>(
     let opts = webrender::RendererOptions {
         resource_override_path: res_path,
         precache_flags: E::PRECACHE_SHADER_FLAGS,
-        device_pixel_ratio,
-        clear_color: Some(ColorF::new(0.3, 0.0, 0.0, 1.0)),
-        //scatter_gpu_cache_updates: false,
+        clear_color: ColorF::new(0.3, 0.0, 0.0, 1.0),
         debug_flags,
+        //allow_texture_swizzling: false,
         ..options.unwrap_or(webrender::RendererOptions::default())
     };
 
     let device_size = {
-        let size = window
-            .get_inner_size()
-            .unwrap()
-            .to_physical(device_pixel_ratio as f64);
+        let size = windowed_context
+            .window()
+            .inner_size();
         DeviceIntSize::new(size.width as i32, size.height as i32)
     };
     let notifier = Box::new(Notifier::new(events_loop.create_proxy()));
@@ -173,16 +180,11 @@ pub fn main_wrapper<E: Example>(
         notifier,
         opts,
         None,
-        device_size,
     ).unwrap();
-    let api = sender.create_api();
-    let document_id = api.add_document(device_size, 0);
+    let mut api = sender.create_api();
+    let document_id = api.add_document(device_size);
 
-    let (external, output) = example.get_image_handlers(&*gl);
-
-    if let Some(output_image_handler) = output {
-        renderer.set_output_image_handler(output_image_handler);
-    }
+    let external = example.get_image_handler(&*gl);
 
     if let Some(external_image_handler) = external {
         renderer.set_external_image_handler(external_image_handler);
@@ -190,12 +192,13 @@ pub fn main_wrapper<E: Example>(
 
     let epoch = Epoch(0);
     let pipeline_id = PipelineId(0, 0);
-    let layout_size = device_size.to_f32() / euclid::TypedScale::new(device_pixel_ratio);
-    let mut builder = DisplayListBuilder::new(pipeline_id, layout_size);
+    let layout_size = device_size.to_f32() / euclid::Scale::new(device_pixel_ratio);
+    let mut builder = DisplayListBuilder::new(pipeline_id);
     let mut txn = Transaction::new();
+    builder.begin();
 
     example.render(
-        &api,
+        &mut api,
         &mut builder,
         &mut txn,
         device_size,
@@ -206,59 +209,62 @@ pub fn main_wrapper<E: Example>(
         epoch,
         Some(ColorF::new(0.3, 0.0, 0.0, 1.0)),
         layout_size,
-        builder.finalize(),
-        true,
+        builder.end(),
     );
     txn.set_root_pipeline(pipeline_id);
-    txn.generate_frame();
+    txn.generate_frame(0, RenderReasons::empty());
     api.send_transaction(document_id, txn);
 
     println!("Entering event loop");
-    events_loop.run_forever(|global_event| {
+    events_loop.run_return(|global_event, _elwt, control_flow| {
         let mut txn = Transaction::new();
         let mut custom_event = true;
 
         let old_flags = debug_flags;
         let win_event = match global_event {
-            winit::Event::WindowEvent { event, .. } => event,
-            _ => return winit::ControlFlow::Continue,
+            winit::event::Event::WindowEvent { event, .. } => event,
+            _ => return,
         };
         match win_event {
-            winit::WindowEvent::CloseRequested => return winit::ControlFlow::Break,
-            // skip high-frequency events
-            winit::WindowEvent::AxisMotion { .. } |
-            winit::WindowEvent::CursorMoved { .. } => return winit::ControlFlow::Continue,
-            winit::WindowEvent::KeyboardInput {
-                input: winit::KeyboardInput {
-                    state: winit::ElementState::Pressed,
+            winit::event::WindowEvent::CloseRequested => {
+                *control_flow = winit::event_loop::ControlFlow::Exit;
+                return;
+            }
+            winit::event::WindowEvent::AxisMotion { .. } |
+            winit::event::WindowEvent::CursorMoved { .. } => {
+                custom_event = example.on_event(
+                    win_event,
+                    windowed_context.window(),
+                    &mut api,
+                    document_id,
+                );
+                // skip high-frequency events from triggering a frame draw.
+                if !custom_event {
+                    return;
+                }
+            },
+            winit::event::WindowEvent::KeyboardInput {
+                input: winit::event::KeyboardInput {
+                    state: winit::event::ElementState::Pressed,
                     virtual_keycode: Some(key),
                     ..
                 },
                 ..
             } => match key {
-                winit::VirtualKeyCode::Escape => return winit::ControlFlow::Break,
-                winit::VirtualKeyCode::P => debug_flags.toggle(DebugFlags::PROFILER_DBG),
-                winit::VirtualKeyCode::O => debug_flags.toggle(DebugFlags::RENDER_TARGET_DBG),
-                winit::VirtualKeyCode::I => debug_flags.toggle(DebugFlags::TEXTURE_CACHE_DBG),
-                winit::VirtualKeyCode::S => debug_flags.toggle(DebugFlags::COMPACT_PROFILER),
-                winit::VirtualKeyCode::T => debug_flags.toggle(DebugFlags::PICTURE_CACHING_DBG),
-                winit::VirtualKeyCode::Q => debug_flags.toggle(
+                winit::event::VirtualKeyCode::Escape => {
+                    *control_flow = winit::event_loop::ControlFlow::Exit;
+                    return;
+                }
+                winit::event::VirtualKeyCode::P => debug_flags.toggle(DebugFlags::PROFILER_DBG),
+                winit::event::VirtualKeyCode::O => debug_flags.toggle(DebugFlags::RENDER_TARGET_DBG),
+                winit::event::VirtualKeyCode::I => debug_flags.toggle(DebugFlags::TEXTURE_CACHE_DBG),
+                winit::event::VirtualKeyCode::T => debug_flags.toggle(DebugFlags::PICTURE_CACHING_DBG),
+                winit::event::VirtualKeyCode::Q => debug_flags.toggle(
                     DebugFlags::GPU_TIME_QUERIES | DebugFlags::GPU_SAMPLE_QUERIES
                 ),
-                winit::VirtualKeyCode::F => debug_flags.toggle(
-                    DebugFlags::NEW_FRAME_INDICATOR | DebugFlags::NEW_SCENE_INDICATOR
-                ),
-                winit::VirtualKeyCode::G => debug_flags.toggle(DebugFlags::GPU_CACHE_DBG),
-                winit::VirtualKeyCode::Key1 => txn.set_document_view(
-                    device_size.into(),
-                    1.0
-                ),
-                winit::VirtualKeyCode::Key2 => txn.set_document_view(
-                    device_size.into(),
-                    2.0
-                ),
-                winit::VirtualKeyCode::M => api.notify_memory_pressure(),
-                winit::VirtualKeyCode::C => {
+                winit::event::VirtualKeyCode::G => debug_flags.toggle(DebugFlags::GPU_CACHE_DBG),
+                winit::event::VirtualKeyCode::M => api.notify_memory_pressure(),
+                winit::event::VirtualKeyCode::C => {
                     let path: PathBuf = "../captures/example".into();
                     //TODO: switch between SCENE/FRAME capture types
                     // based on "shift" modifier, when `glutin` is updated.
@@ -268,14 +274,16 @@ pub fn main_wrapper<E: Example>(
                 _ => {
                     custom_event = example.on_event(
                         win_event,
-                        &api,
+                        windowed_context.window(),
+                        &mut api,
                         document_id,
                     )
                 },
             },
             other => custom_event = example.on_event(
                 other,
-                &api,
+                windowed_context.window(),
+                &mut api,
                 document_id,
             ),
         };
@@ -285,10 +293,11 @@ pub fn main_wrapper<E: Example>(
         }
 
         if custom_event {
-            let mut builder = DisplayListBuilder::new(pipeline_id, layout_size);
+            let mut builder = DisplayListBuilder::new(pipeline_id);
+            builder.begin();
 
             example.render(
-                &api,
+                &mut api,
                 &mut builder,
                 &mut txn,
                 device_size,
@@ -299,20 +308,19 @@ pub fn main_wrapper<E: Example>(
                 epoch,
                 Some(ColorF::new(0.3, 0.0, 0.0, 1.0)),
                 layout_size,
-                builder.finalize(),
-                true,
+                builder.end(),
             );
-            txn.generate_frame();
+            txn.generate_frame(0, RenderReasons::empty());
         }
         api.send_transaction(document_id, txn);
 
         renderer.update();
-        renderer.render(device_size).unwrap();
+        renderer.render(device_size, 0).unwrap();
         let _ = renderer.flush_pipeline_info();
         example.draw_custom(&*gl);
-        window.swap_buffers().ok();
+        windowed_context.swap_buffers().ok();
 
-        winit::ControlFlow::Continue
+        *control_flow = winit::event_loop::ControlFlow::Wait;
     });
 
     renderer.deinit();
